@@ -4,7 +4,7 @@
     
     Copyright (C) 1998 David A. Hinds -- dhinds@hyper.stanford.edu
 
-    3c589_cs.c 1.122 1999/05/14 17:30:54
+    3c589_cs.c 1.128 1999/07/30 15:17:50
 
     The network driver code is based on Donald Becker's 3c589 code:
     
@@ -55,8 +55,9 @@
 #define EL3_TIMER	0x0a
 #define EL3_CMD		0x0e
 #define EL3_STATUS	0x0e
-#define ID_PORT		0x100
-#define	EEPROM_READ	0x80
+
+#define EEPROM_READ	0x0080
+#define EEPROM_BUSY	0x8000
 
 #define EL3WINDOW(win_num) outw(SelectWindow + (win_num), ioaddr + EL3_CMD)
 
@@ -93,6 +94,7 @@ enum RxFilter {
 #define WN0_IRQ		0x08	/* Window 0: Set IRQ line in bits 12-15. */
 #define WN4_MEDIA	0x0A	/* Window 4: Various transcvr/media bits. */
 #define MEDIA_TP	0x00C0	/* Enable link beat and jabber for 10baseT. */
+#define MEDIA_LED	0x0001	/* Enable link light on 3C589E cards. */
 
 /* Time in jiffies before concluding Tx hung */
 #define TX_TIMEOUT	((400*HZ)/1000)
@@ -107,14 +109,14 @@ struct el3_private {
     u_long	last_irq;
 };
 
-static char *if_names[] = { "Auto", "10baseT", "10base2", "AUI" };
+static char *if_names[] = { "auto", "10baseT", "10base2", "AUI" };
 
 #ifdef PCMCIA_DEBUG
 static int pc_debug = PCMCIA_DEBUG;
 MODULE_PARM(pc_debug, "i");
 #define DEBUG(n, args...) if (pc_debug>(n)) printk(KERN_DEBUG args)
 static char *version =
-"3c589_cs.c 1.122 1999/05/14 17:30:54 (David Hinds)";
+"3c589_cs.c 1.128 1999/07/30 15:17:50 (David Hinds)";
 #else
 #define DEBUG(n, args...)
 #endif
@@ -445,14 +447,15 @@ static void tc589_config(dev_link_t *link)
     else
 	printk(KERN_NOTICE "3c589_cs: invalid if_port requested\n");
     
-    printk(KERN_INFO "%s: 3Com 3c%s, port %#3lx, irq %d, %s port, "
-	   "hw_addr ", dev->name, (multi ? "562" : "589"),
-	   dev->base_addr, dev->irq, if_names[dev->if_port]);
+    printk(KERN_INFO "%s: 3Com 3c%s, io %#3lx, irq %d, hw_addr ",
+	   dev->name, (multi ? "562" : "589"), dev->base_addr,
+	   dev->irq);
     for (i = 0; i < 6; i++)
 	printk("%02X%s", dev->dev_addr[i], ((i<5) ? ":" : "\n"));
     i = inl(ioaddr);
-    printk(KERN_INFO "  %dK FIFO split %s Rx:Tx\n",
-	   (i & 7) ? 32 : 8, ram_split[(i >> 16) & 3]);
+    printk(KERN_INFO "  %dK FIFO split %s Rx:Tx, %s xcvr\n",
+	   (i & 7) ? 32 : 8, ram_split[(i >> 16) & 3],
+	   if_names[dev->if_port]);
     return;
 
 cs_failed:
@@ -575,7 +578,7 @@ static ushort read_eeprom(short ioaddr, int index)
     outw(EEPROM_READ + index, ioaddr + 10);
     /* Reading the eeprom takes 162 us */
     for (i = 1620; i >= 0; i--)
-	if ((inw(ioaddr + 10) & 0x8000) == 0)
+	if ((inw(ioaddr + 10) & EEPROM_BUSY) == 0)
 	    break;
     return inw(ioaddr + 12);
 }
@@ -599,10 +602,12 @@ static void tc589_set_xcvr(struct device *dev, int if_port)
     outw((if_port == 2) ? StartCoax : StopCoax, ioaddr + EL3_CMD);
     /* 10baseT interface, enable link beat and jabber check. */
     EL3WINDOW(4);
-    outw((if_port < 2) ? MEDIA_TP : 0, ioaddr + WN4_MEDIA);
+    outw(MEDIA_LED | ((if_port < 2) ? MEDIA_TP : 0), ioaddr + WN4_MEDIA);
     EL3WINDOW(1);
-    lp->media_status = (if_port < 2) ? 0x8800 : 0x4800;
-    lp->last_irq = jiffies;
+    if (if_port == 2)
+	lp->media_status = ((dev->if_port == 0) ? 0x8000 : 0x4000);
+    else
+	lp->media_status = ((dev->if_port == 0) ? 0x4010 : 0x8800);
 }
 
 static void dump_status(struct device *dev)
@@ -893,10 +898,10 @@ static void media_check(u_long arg)
     struct device *dev = (struct device *)(arg);
     struct el3_private *lp = (struct el3_private *)dev->priv;
     int ioaddr = dev->base_addr;
-    u_short media;
+    u_short media, errs;
     u_long flags;
 
-    if (dev->start == 0) return;
+    if (dev->start == 0) goto reschedule;
     
     EL3WINDOW(1);
     /* Check for pending interrupt with expired latency timer: with
@@ -921,8 +926,18 @@ static void media_check(u_long arg)
     media = inw(ioaddr+WN4_MEDIA) & 0xc810;
 
     /* Ignore collisions unless we've had no irq's recently */
-    if (jiffies - lp->last_irq < HZ)
+    if (jiffies - lp->last_irq < HZ) {
 	media &= ~0x0010;
+    } else {
+	/* Try harder to detect carrier errors */
+	EL3WINDOW(6);
+	outw(StatsDisable, ioaddr + EL3_CMD);
+	errs = inb(ioaddr + 0);
+	outw(StatsEnable, ioaddr + EL3_CMD);
+	lp->stats.tx_carrier_errors += errs;
+	if (errs || (lp->media_status & 0x0010)) media |= 0x0010;
+    }
+
     if (media != lp->media_status) {
 	if ((media & lp->media_status & 0x8000) &&
 	    ((lp->media_status ^ media) & 0x0800))
@@ -930,8 +945,8 @@ static void media_check(u_long arg)
 		   (lp->media_status & 0x0800 ? "lost" : "found"));
 	else if ((media & lp->media_status & 0x4000) &&
 		 ((lp->media_status ^ media) & 0x0010))
-	    printk(KERN_INFO "%s: cable %s\n", dev->name,
-		   (lp->media_status & 0x0010 ? "fixed" : "problem"));
+	    printk(KERN_INFO "%s: coax cable %s\n", dev->name,
+		   (lp->media_status & 0x0010 ? "ok" : "problem"));
 	if (dev->if_port == 0) {
 	    if (media & 0x8000) {
 		if (media & 0x0800)
@@ -939,8 +954,7 @@ static void media_check(u_long arg)
 			   dev->name);
 		else
 		    tc589_set_xcvr(dev, 2);
-	    }
-	    if (media & 0x4000) {
+	    } else if (media & 0x4000) {
 		if (media & 0x0010)
 		    tc589_set_xcvr(dev, 1);
 		else
@@ -953,6 +967,8 @@ static void media_check(u_long arg)
     
     EL3WINDOW(1);
     restore_flags(flags);
+
+reschedule:
     lp->media.expires = RUN_AT(HZ);
     add_timer(&lp->media);
 }
@@ -1121,7 +1137,7 @@ static int el3_close(struct device *dev)
 	else if (dev->if_port == 1) {
 	    /* Disable link beat and jabber */
 	    EL3WINDOW(4);
-	    outw(inw(ioaddr + WN4_MEDIA) & ~MEDIA_TP, ioaddr + WN4_MEDIA);
+	    outw(0, ioaddr + WN4_MEDIA);
 	}
 	
 	/* Switching back to window 0 disables the IRQ. */
