@@ -5,10 +5,10 @@
     This driver supports the Adaptec AHA-1460, the New Media Bus
     Toaster, and the New Media Toast & Jam.
     
-    aha152x_cs.c 1.43 1998/12/24 20:33:04
+    aha152x_cs.c 1.45 1999/07/30 03:49:13
 
     The contents of this file are subject to the Mozilla Public
-    License Version 1.0 (the "License"); you may not use this file
+    License Version 1.1 (the "License"); you may not use this file
     except in compliance with the License. You may obtain a copy of
     the License at http://www.mozilla.org/MPL/
 
@@ -34,8 +34,8 @@
 #include <linux/ioport.h>
 #include <scsi/scsi.h>
 #include <linux/major.h>
+#include <linux/blk.h>
 
-#include BLK_DEV_HDR
 #include "drivers/scsi/scsi.h"
 #include "drivers/scsi/hosts.h"
 #include <scsi/scsi_ioctl.h>
@@ -56,7 +56,7 @@ static int pc_debug = PCMCIA_DEBUG;
 MODULE_PARM(pc_debug, "i");
 #define DEBUG(n, args...) if (pc_debug>(n)) printk(KERN_DEBUG args)
 static char *version =
-"aha152x_cs.c 1.43 1998/12/24 20:33:04 (David Hinds)";
+"aha152x_cs.c 1.45 1999/07/30 03:49:13 (David Hinds)";
 #else
 #define DEBUG(n, args...)
 #endif
