@@ -2,10 +2,10 @@
 
     A driver for PCMCIA IDE/ATA disk cards
 
-    ide_cs.c 1.16 1999/05/17 04:40:27
+    ide_cs.c 1.18 1999/07/20 15:59:40
 
     The contents of this file are subject to the Mozilla Public
-    License Version 1.0 (the "License"); you may not use this file
+    License Version 1.1 (the "License"); you may not use this file
     except in compliance with the License. You may obtain a copy of
     the License at http://www.mozilla.org/MPL/
 
@@ -50,7 +50,7 @@ static int pc_debug = PCMCIA_DEBUG;
 MODULE_PARM(pc_debug, "i");
 #define DEBUG(n, args...) if (pc_debug>(n)) printk(KERN_DEBUG args)
 static char *version =
-"ide_cs.c 1.16 1999/05/17 04:40:27 (David Hinds)";
+"ide_cs.c 1.18 1999/07/20 15:59:40 (David Hinds)";
 #else
 #define DEBUG(n, args...)
 #endif
@@ -253,6 +253,7 @@ void ide_config(dev_link_t *link)
 
     /* Not sure if this is right... look up the current Vcc */
     CS_CHECK(GetConfigurationInfo, handle, &conf);
+    link->conf.Vcc = conf.Vcc;
     
     pass = io_base = ctl_base = 0;
     tuple.DesiredTuple = CISTPL_CFTABLE_ENTRY;
@@ -262,12 +263,16 @@ void ide_config(dev_link_t *link)
 	CFG_CHECK(GetTupleData, handle, &tuple);
 	CFG_CHECK(ParseTuple, handle, &tuple, &parse);
 
-	if (cfg->vcc.present & (1<<CISTPL_POWER_VNOM))
-	    link->conf.Vcc = cfg->vcc.param[CISTPL_POWER_VNOM]/10000;
-	else if (dflt.vcc.present & (1<<CISTPL_POWER_VNOM))
-	    link->conf.Vcc = dflt.vcc.param[CISTPL_POWER_VNOM]/10000;
-	if ((!pass) && (link->conf.Vcc != conf.Vcc))
-	    goto next_entry;
+	/* Check for matching Vcc, unless we're desperate */
+	if (!pass) {
+	    if (cfg->vcc.present & (1<<CISTPL_POWER_VNOM)) {
+		if (conf.Vcc != cfg->vcc.param[CISTPL_POWER_VNOM]/10000)
+		    goto next_entry;
+	    } else if (dflt.vcc.present & (1<<CISTPL_POWER_VNOM)) {
+		if (conf.Vcc != dflt.vcc.param[CISTPL_POWER_VNOM]/10000)
+		    goto next_entry;
+	    }
+	}
 	
 	if (cfg->vpp1.present & (1<<CISTPL_POWER_VNOM))
 	    link->conf.Vpp1 = link->conf.Vpp2 =
@@ -306,6 +311,7 @@ void ide_config(dev_link_t *link)
 	    CS_CHECK(GetNextTuple, handle, &tuple);
 	} else if (CardServices(GetNextTuple, handle, &tuple) != 0) {
 	    CS_CHECK(GetFirstTuple, handle, &tuple);
+	    memset(&dflt, 0, sizeof(dflt));
 	    pass++;
 	}
     }
