@@ -2,10 +2,10 @@
 
     A direct memory interface driver for CardBus cards
 
-    memory_cb.c 1.4 1998/11/05 07:13:23
+    memory_cb.c 1.6 1999/07/30 03:49:13
 
     The contents of this file are subject to the Mozilla Public
-    License Version 1.0 (the "License"); you may not use this file
+    License Version 1.1 (the "License"); you may not use this file
     except in compliance with the License. You may obtain a copy of
     the License at http://www.mozilla.org/MPL/
 
@@ -39,7 +39,7 @@ static int pc_debug = PCMCIA_DEBUG;
 MODULE_PARM(pc_debug, "i");
 #define DEBUG(n, args...) if (pc_debug>(n)) printk(KERN_DEBUG args)
 static char *version =
-"memory_cb.c 1.4 1998/11/05 07:13:23 (David Hinds)";
+"memory_cb.c 1.6 1999/07/30 03:49:13 (David Hinds)";
 #else
 #define DEBUG(n, args...)
 #endif
@@ -197,7 +197,7 @@ static FS_SIZE_T memory_read FOPS(struct inode *inode,
 }
 
 static FS_SIZE_T memory_write FOPS(struct inode *inode,
-				   struct file *file, CONST char *buf,
+				   struct file *file, const char *buf,
 				   U_FS_SIZE_T count, loff_t *ppos)
 {
     int minor = MINOR(F_INODE(file)->i_rdev);
