@@ -7,10 +7,10 @@
     card's attribute and common memory.  It includes character
     and block device support.
 
-    memory_cs.c 1.53 1998/11/18 08:01:13
+    memory_cs.c 1.59 1999/07/30 05:36:15
 
     The contents of this file are subject to the Mozilla Public
-    License Version 1.0 (the "License"); you may not use this file
+    License Version 1.1 (the "License"); you may not use this file
     except in compliance with the License. You may obtain a copy of
     the License at http://www.mozilla.org/MPL/
 
@@ -42,6 +42,10 @@
 #include <asm/segment.h>
 #include <stdarg.h>
 
+#if (LINUX_VERSION_CODE >= VERSION(2,3,3))
+#include <linux/blkpg.h>
+#endif
+
 #include <pcmcia/version.h>
 #include <pcmcia/cs_types.h>
 #include <pcmcia/cs.h>
@@ -68,14 +72,14 @@ static int major_dev = 0;
 #define MINOR_NR(dev,dir,attr,rgn) \
 (((dev)<<4)+((dir)<<3)+((attr)<<2)+(rgn))
 
-#include BLK_DEV_HDR
+#include <linux/blk.h>
 
 #ifdef PCMCIA_DEBUG
 static int pc_debug = PCMCIA_DEBUG;
 MODULE_PARM(pc_debug, "i");
 #define DEBUG(n, args...) if (pc_debug>(n)) printk(KERN_DEBUG args)
 static char *version =
-"memory_cs.c 1.53 1998/11/18 08:01:13 (David Hinds)";
+"memory_cs.c 1.59 1999/07/30 05:36:15 (David Hinds)";
 #else
 #define DEBUG(n, args...)
 #endif
@@ -947,12 +951,6 @@ static int memory_ioctl(struct inode *inode, struct file *file,
 	    put_user(minor_dev->region.RegionSize/SECTOR_SIZE,
 		     (long *)arg);
 	break;
-    case BLKFLSBUF:
-	if (!suser()) return -EACCES;
-	if (!(inode->i_rdev)) return -EINVAL;
-	fsync_dev(inode->i_rdev);
-	invalidate_buffers(inode->i_rdev);
-	break;
     case MEMGETINFO:
 	if (!IS_DIRECT(minor)) {
 	    copy_to_user((region_info_t *)arg, &minor_dev->region,
@@ -966,6 +964,20 @@ static int memory_ioctl(struct inode *inode, struct file *file,
 	    ret = memory_erase(minor, erase.Offset, erase.Size);
 	} else ret = -EINVAL;
 	break;
+#if (LINUX_VERSION_CODE < VERSION(2,3,3))
+    case BLKFLSBUF:
+	if (!suser()) return -EACCES;
+	if (!(inode->i_rdev)) return -EINVAL;
+	fsync_dev(inode->i_rdev);
+	invalidate_buffers(inode->i_rdev);
+	break;
+#else
+    case BLKROSET:
+    case BLKROGET:
+    case BLKFLSBUF:
+	ret = blk_ioctl(inode->i_rdev, cmd, arg);
+	break;
+#endif
     default:
 	ret = -EINVAL;
     }
@@ -1048,7 +1060,7 @@ static void do_memory_request(void)
     do {
 	INIT_REQUEST;
 
-	minor = MINOR(DEVICE(CURRENT));
+	minor = MINOR(CURRENT->rq_dev);
 	link = dev_table[DEVICE_NR(minor)];
 	dev = (memory_dev_t *)link->priv;
 
@@ -1111,11 +1123,12 @@ int init_module(void)
 		unregister_chrdev(i, "memory");
 	}
     }
-    if (i == 0)
+    if (i == 0) {
 	printk(KERN_NOTICE "memory_cs: unable to grab a device #\n");
-    else
-	major_dev = i;
+	return -ENODEV;
+    }
 
+    major_dev = i;
     blk_dev[major_dev].request_fn = DEVICE_REQUEST;
     for (i = 0; i < MINOR_NR(MAX_DEV, 0, 0, 0); i++)
 	memory_blocksizes[i] = 1024;
@@ -1134,6 +1147,8 @@ void cleanup_module(void)
     if (major_dev != 0) {
 	unregister_chrdev(major_dev, "memory");
 	unregister_blkdev(major_dev, "memory");
+	blk_dev[major_dev].request_fn = NULL;
+	blksize_size[major_dev] = NULL;
     }
     for (i = 0; i < MAX_DEV; i++) {
 	link = dev_table[i];
