@@ -2,10 +2,10 @@
 
     A driver for PCMCIA serial devices
 
-    serial_cs.c 1.103 1999/02/13 06:47:03
+    serial_cs.c 1.106 1999/07/20 16:00:02
 
     The contents of this file are subject to the Mozilla Public
-    License Version 1.0 (the "License"); you may not use this file
+    License Version 1.1 (the "License"); you may not use this file
     except in compliance with the License. You may obtain a copy of
     the License at http://www.mozilla.org/MPL/
 
@@ -48,7 +48,7 @@ static int pc_debug = PCMCIA_DEBUG;
 MODULE_PARM(pc_debug, "i");
 #define DEBUG(n, args...) if (pc_debug>(n)) printk(KERN_DEBUG args)
 static char *version =
-"serial_cs.c 1.103 1999/02/13 06:47:03 (David Hinds)";
+"serial_cs.c 1.106 1999/07/20 16:00:02 (David Hinds)";
 #else
 #define DEBUG(n, args...)
 #endif
@@ -84,7 +84,8 @@ static multi_id_t multi_id[] = {
     { MANFID_QUATECH, PRODID_QUATECH_DUAL_RS232_D1, 2 },
     { MANFID_QUATECH, PRODID_QUATECH_QUAD_RS232, 4 },
     { MANFID_SOCKET, PRODID_SOCKET_DUAL_RS232, 2 },
-    { MANFID_INTEL, PRODID_INTEL_DUAL_RS232, 2 }
+    { MANFID_INTEL, PRODID_INTEL_DUAL_RS232, 2 },
+    { MANFID_NATINST, PRODID_NATINST_QUAD_RS232, 4 }
 };
 #define MULTI_COUNT (sizeof(multi_id)/sizeof(multi_id_t))
 
@@ -302,6 +303,7 @@ static int simple_config(dev_link_t *link)
 	if (info->slave)
 	    return setup_serial(info, port, config.AssignedIRQ);
     }
+    link->conf.Vcc = config.Vcc;
     
     /* First pass: look for a config entry that looks normal. */
     tuple.TupleData = (cisdata_t *)buf;
