@@ -1,8 +1,8 @@
 /*
- * ss.h 1.20 1999/04/08 06:04:16
+ * ss.h 1.23 1999/07/20 16:08:01
  *
  * The contents of this file are subject to the Mozilla Public License
- * Version 1.0 (the "License"); you may not use this file except in
+ * Version 1.1 (the "License"); you may not use this file except in
  * compliance with the License. You may obtain a copy of the License
  * at http://www.mozilla.org/MPL/
  *
@@ -48,12 +48,14 @@ typedef struct socket_cap_t {
     u_int	map_size;
     u_char	pci_irq;
     u_char	cardbus;
+    struct pci_bus *cb_bus;
     struct bus_operations *bus;
 } socket_cap_t;
 
 /* InquireSocket capabilities */
 #define SS_CAP_PAGE_REGS	0x0001
 #define SS_CAP_VIRTUAL_BUS	0x0002
+#define SS_CAP_MEM_ALIGN	0x0004
 #define SS_CAP_PCCARD		0x4000
 #define SS_CAP_CARDBUS		0x8000
 
