@@ -2,10 +2,10 @@
   
     Cardbus device configuration
     
-    cardbus.c 1.47 1999/04/14 05:52:07
+    cardbus.c 1.52 1999/07/20 16:01:21
 
     The contents of this file are subject to the Mozilla Public
-    License Version 1.0 (the "License"); you may not use this file
+    License Version 1.1 (the "License"); you may not use this file
     except in compliance with the License. You may obtain a copy of
     the License at http://www.mozilla.org/MPL/
 
@@ -84,7 +84,7 @@ static int pc_debug = PCMCIA_DEBUG;
 #define PCDATA_INDICATOR	0x0015
 
 #ifdef __LINUX__
-#if (LINUX_VERSION_CODE >= VERSION(2,1,90))
+#if (LINUX_VERSION_CODE >= VERSION(2,1,103))
 #define NEW_LINUX_PCI
 #endif
 #endif
@@ -126,16 +126,16 @@ static int check_rom(u_char *b, u_long len)
 	    (readb(b+ROM_DATA_PTR+1) << 8);
 	sz = 512 * (readb(b+data+PCDATA_IMAGE_SZ) +
 		    (readb(b+data+PCDATA_IMAGE_SZ+1) << 8));
-	if (sz == 0) break;
 	DEBUG(0, ("  image %d: 0x%06x-0x%06x, signature %c%c%c%c\n",
 		  img, ofs, ofs+sz-1,
 		  readb(b+data+PCDATA_SIGNATURE),
 		  readb(b+data+PCDATA_SIGNATURE+1),
 		  readb(b+data+PCDATA_SIGNATURE+2),
 		  readb(b+data+PCDATA_SIGNATURE+3)));
-	b += sz; ofs += sz; img++;
+	ofs += sz; img++;
 	if ((readb(b+data+PCDATA_INDICATOR) & 0x80) ||
-	    (ofs >= len)) break;
+	    (sz == 0) || (ofs >= len)) break;
+	b += sz;
     }
     return img;
 }
@@ -150,9 +150,9 @@ static u_int xlate_rom_addr(u_char *b, u_int addr)
 	data = readb(b+ROM_DATA_PTR) + (readb(b+ROM_DATA_PTR+1) << 8);
 	sz = 512 * (readb(b+data+PCDATA_IMAGE_SZ) +
 		    (readb(b+data+PCDATA_IMAGE_SZ+1) << 8));
-	if (sz == 0) break;
+	if ((sz == 0) || (readb(b+data+PCDATA_INDICATOR) & 0x80))
+	    break;
 	b += sz; ofs += sz; img++;
-	if (readb(b+data+PCDATA_INDICATOR) & 0x80) break;
     }
     return 0;
 }
@@ -192,7 +192,7 @@ int cb_setup_cis_mem(socket_info_t *s, int space)
     sz &= PCI_BASE_ADDRESS_MEM_MASK;
     sz = FIND_FIRST_BIT(sz);
     if (sz < PAGE_SIZE) sz = PAGE_SIZE;
-    if (find_mem_region(&base, sz, "cb_enabler", 0) != 0) {
+    if (find_mem_region(&base, sz, "cb_enabler", sz, 0) != 0) {
 	printk(KERN_NOTICE "cs: could not allocate %dK memory for"
 	       " CardBus socket %d\n", sz/1024, s->sock);
 	return CS_OUT_OF_RESOURCE;
@@ -232,7 +232,7 @@ void cb_release_cis_mem(socket_info_t *s)
 	s->ss_entry(s->sock, SS_SetBridge, m);
 	pci_writeb(s->cap.cardbus, 0, PCI_COMMAND, 0);
 	pci_writel(s->cap.cardbus, 0, br, 0);
-	release_mem_region(m->start, m->stop - m->start + 1);
+	vacate_mem_region(m->start, m->stop - m->start + 1);
 	m->start = 0;
     }
 }
@@ -256,8 +256,13 @@ void read_cb_mem(socket_info_t *s, u_char fn, int space,
 	    memset(ptr, 0xff, len);
 	    return;
 	}
-	if (space == 7)
+	if (space == 7) {
 	    addr = xlate_rom_addr(s->cb_cis_virt, addr);
+	    if (addr == 0) {
+		memset(ptr, 0xff, len);
+		return;
+	    }
+	}
 	if (s->cb_cis_virt != NULL)
 	    for (; len; addr++, ptr++, len--)
 		*(u_char *)ptr = readb(s->cb_cis_virt+addr);
@@ -304,11 +309,8 @@ int cb_config(socket_info_t *s)
     s->cb_config = c;
 
 #ifdef NEW_LINUX_PCI
-    s->pci_bus = kmalloc(sizeof(struct pci_bus), GFP_KERNEL);
-    memset(s->pci_bus, 0, sizeof(struct pci_bus));
-    s->pci_bus->number = bus;
     for (i = 0; i < fn; i++) {
-	c[i].dev.bus = s->pci_bus;
+	c[i].dev.bus = s->cap.cb_bus;
 	if (i < fn-1) {
 	    c[i].dev.sibling = c[i].dev.next = &c[i+1].dev;
 	}
@@ -317,8 +319,9 @@ int cb_config(socket_info_t *s)
 	pci_readl(bus, i, PCI_CLASS_REVISION, &c[i].dev.class);
 	c[i].dev.class >>= 8;
 	pci_readb(bus, i, PCI_HEADER_TYPE, &j);
-	c[i].dev.hdr_type = j;	
+	c[i].dev.hdr_type = j;
     }
+    s->cap.cb_bus->devices = &c[0].dev;
 #endif
     
     /* Determine IO and memory space needs */
@@ -366,7 +369,8 @@ int cb_config(socket_info_t *s)
     s->win[0].size = num[B_M1];
     s->win[0].base = 0;
     if (num[B_M1]) {
-	if (find_mem_region(&s->win[0].base, num[B_M1], name, 0) != 0) {
+	if (find_mem_region(&s->win[0].base, num[B_M1],
+			    name, num[B_M1], 0) != 0) {
 	    printk(KERN_NOTICE "cs: could not allocate %dK memory for"
 		   " CardBus socket %d\n", num[B_M1]/1024, s->sock);
 	    goto failed;
@@ -376,7 +380,8 @@ int cb_config(socket_info_t *s)
     s->win[1].size = num[B_M2];
     s->win[1].base = 0;
     if (num[B_M2]) {
-	if (find_mem_region(&s->win[1].base, num[B_M2], name, 0) != 0) {
+	if (find_mem_region(&s->win[1].base, num[B_M2],
+			    name, num[B_M2], 0) != 0) {
 	    printk(KERN_NOTICE "cs: could not allocate %dK memory for"
 		   " CardBus socket %d\n", num[B_M2]/1024, s->sock);
 	    goto failed;
@@ -463,11 +468,11 @@ void cb_release(socket_info_t *s)
     DEBUG(0, ("cs: cb_release(bus %d)\n", s->cap.cardbus));
     
     if (s->win[0].size > 0)
-	release_mem_region(s->win[0].base, s->win[0].size);
+	vacate_mem_region(s->win[0].base, s->win[0].size);
     if (s->win[1].size > 0)
-	release_mem_region(s->win[1].base, s->win[1].size);
+	vacate_mem_region(s->win[1].base, s->win[1].size);
     if (s->io[0].NumPorts > 0)
-	release_region(s->io[0].BasePort, s->io[0].NumPorts);
+	vacate_region(s->io[0].BasePort, s->io[0].NumPorts);
     s->io[0].NumPorts = 0;
 #ifdef CONFIG_ISA
     if ((c[0].dev.irq != 0) && (c[0].dev.irq != s->cap.pci_irq))
@@ -475,10 +480,6 @@ void cb_release(socket_info_t *s)
 #endif
     kfree(s->cb_config);
     s->cb_config = NULL;
-#ifdef NEW_LINUX_PCI
-    kfree(s->pci_bus);
-    s->pci_bus = NULL;
-#endif
 }
 
 /*=====================================================================
@@ -561,9 +562,7 @@ void cb_enable(socket_info_t *s)
     /* Link into PCI device chain */
     c[s->functions-1].dev.next = pci_devices;
     pci_devices = &c[0].dev;
-#if (LINUX_VERSION_CODE >= VERSION(2,1,103))
-    /* pci_proc_attach_device(&c[0].dev); */
-#endif
+    pci_proc_attach_device(&c[0].dev);
 #endif
 }
 
@@ -585,14 +584,13 @@ void cb_disable(socket_info_t *s)
     cb_config_t *c = s->cb_config;
     
     /* Unlink from PCI device chain */
-#if (LINUX_VERSION_CODE >= VERSION(2,1,103))
-    /* pci_proc_detach_device(&c[0].dev); */
-#endif
+    pci_proc_detach_device(&c[0].dev);
     for (p = &pci_devices; *p; p = &((*p)->next))
 	if (*p == &c[0].dev) break;
     for (q = *p; q; q = q->next)
 	if (q->bus != (*p)->bus) break;
     if (*p) *p = q;
+    s->cap.cb_bus->devices = NULL;
 #endif
     
     DEBUG(0, ("cs: cb_disable(bus %d)\n", s->cap.cardbus));
