@@ -2,10 +2,10 @@
 
     PCMCIA Card Information Structure parser
 
-    cistpl.c 1.63 1999/04/27 15:54:07
+    cistpl.c 1.68 1999/07/20 16:01:23
 
     The contents of this file are subject to the Mozilla Public
-    License Version 1.0 (the "License"); you may not use this file
+    License Version 1.1 (the "License"); you may not use this file
     except in compliance with the License. You may obtain a copy of
     the License at http://www.mozilla.org/MPL/
 
@@ -34,6 +34,7 @@
 #include <linux/mm.h>
 #include <linux/sched.h>
 #include <linux/pci.h>
+#include <linux/ioport.h>
 #include <asm/io.h>
 #endif
 
@@ -87,7 +88,7 @@ void read_cis_mem(socket_info_t *s, int attr, u_int addr,
 	memset(ptr, 0xff, len);
 	return;
     }
-    mem->flags &= ~MAP_ATTRIB;
+    mem->flags |= MAP_ACTIVE; mem->flags &= ~MAP_ATTRIB;
     if (attr) { mem->flags |= MAP_ATTRIB; inc++; addr *= 2; }
     sys = s->cis_virt + (addr & (s->cap.map_size-1));
     mem->card_start = addr & ~(s->cap.map_size-1);
@@ -199,11 +200,12 @@ int setup_cis_mem(socket_info_t *s)
 	s->cis_mem.sys_start = 0;
 	vs = NULL;
 	if (find_mem_region(&s->cis_mem.sys_start, s->cap.map_size,
-			    "card services", low) != 0) {
+			    "card services", s->cap.map_size, low)) {
 	    printk(KERN_NOTICE "cs: unable to map card memory!\n");
 	    return CS_OUT_OF_RESOURCE;
 	}
 	s->cis_mem.sys_stop = s->cis_mem.sys_start+s->cap.map_size-1;
+	s->cis_mem.flags |= MAP_ACTIVE;
 	s->cis_virt = bus_ioremap(s->cap.bus, s->cis_mem.sys_start,
 				  s->cap.map_size);
     }
@@ -213,7 +215,9 @@ int setup_cis_mem(socket_info_t *s)
 void release_cis_mem(socket_info_t *s)
 {
     if (s->cis_mem.sys_start != 0) {
-	release_mem_region(s->cis_mem.sys_start, s->cap.map_size);
+	s->cis_mem.flags &= ~MAP_ACTIVE;
+	s->ss_entry(s->sock, SS_SetMemMap, &s->cis_mem);
+	vacate_mem_region(s->cis_mem.sys_start, s->cap.map_size);
 	bus_iounmap(s->cap.bus, s->cis_virt);
 	s->cis_mem.sys_start = 0;
     }
