@@ -2,10 +2,10 @@
 
     PCMCIA Card Services -- core services
 
-    cs.c 1.216 1999/05/14 17:36:21
+    cs.c 1.222 1999/07/20 16:01:24
     
     The contents of this file are subject to the Mozilla Public
-    License Version 1.0 (the "License"); you may not use this file
+    License Version 1.1 (the "License"); you may not use this file
     except in compliance with the License. You may obtain a copy of
     the License at http://www.mozilla.org/MPL/
 
@@ -60,7 +60,7 @@ static int handle_apm_event(apm_event_t event);
 int pc_debug = PCMCIA_DEBUG;
 MODULE_PARM(pc_debug, "i");
 static const char *version =
-"cs.c 1.216 1999/05/14 17:36:21 (David Hinds)";
+"cs.c 1.222 1999/07/20 16:01:24 (David Hinds)";
 #endif
 
 #ifdef __BEOS__
@@ -300,7 +300,7 @@ int register_ss_entry(int nsock, ss_entry_t ss_entry)
 	s->shutdown.data = sockets;
 	s->shutdown.function = &shutdown_socket;
 	/* base address = 0, map = 0 */
-	s->cis_mem.flags = MAP_ACTIVE;
+	s->cis_mem.flags = 0;
 	s->cis_mem.speed = cis_speed;
 	s->erase_busy.next = s->erase_busy.prev = &s->erase_busy;
 	
@@ -733,7 +733,7 @@ static void release_io_space(socket_info_t *s, ioaddr_t base,
 			     ioaddr_t num)
 {
     int i;
-    release_region(base, num);
+    vacate_region(base, num);
     for (i = 0; i < MAX_IO_WIN; i++) {
 	if ((s->io[i].BasePort <= base) &&
 	    (s->io[i].BasePort+s->io[i].NumPorts >= base+num)) {
@@ -1121,14 +1121,16 @@ static int get_status(client_handle_t handle, cs_status_t *status)
 
 static int map_mem_page(window_handle_t win, memreq_t *req)
 {
+    socket_info_t *s;
     if ((win == NULL) || (win->magic != WINDOW_MAGIC))
 	return CS_BAD_HANDLE;
     if (req->Page != 0)
 	return CS_BAD_PAGE;
 
+    s = win->sock;
     win->ctl.card_start = req->CardOffset;
-    win->sock->ss_entry(win->sock->sock, SS_SetMemMap, &win->ctl);
-    
+    if (s->ss_entry(s->sock, SS_SetMemMap, &win->ctl) != 0)
+	return CS_BAD_OFFSET;
     return CS_SUCCESS;
 } /* map_mem_page */
 
@@ -1163,11 +1165,8 @@ static int modify_configuration(client_handle_t handle,
 	s->ss_entry(s->sock, SS_SetSocket, &s->socket);
     }
 
-    if (mod->Attributes & CONF_VCC_CHANGE_VALID) {
-	c->Vcc = s->socket.Vcc = mod->Vcc;
-	if (s->ss_entry(s->sock, SS_SetSocket, &s->socket))
-	    return CS_BAD_VCC;
-    }
+    if (mod->Attributes & CONF_VCC_CHANGE_VALID)
+	return CS_BAD_VCC;
 
     /* We only allow changing Vpp1 and Vpp2 to the same value */
     if ((mod->Attributes & CONF_VPP1_CHANGE_VALID) &&
@@ -1450,7 +1449,7 @@ static int release_window(window_handle_t win)
     s->state &= ~SOCKET_WIN_REQ(win->index);
 
     /* Release system memory */
-    release_mem_region(win->base, win->size);
+    vacate_mem_region(win->base, win->size);
     win->handle->state &= ~CLIENT_WIN_REQ(win->index);
 
     win->magic = 0;
@@ -1494,11 +1493,9 @@ static int request_configuration(client_handle_t handle,
     if (c->state & CONFIG_LOCKED)
 	return CS_CONFIGURATION_LOCKED;
 
-    /* Do power control */
-    s->socket.Vcc = req->Vcc;
-    if (s->ss_entry(s->sock, SS_SetSocket, &s->socket))
+    /* Do power control.  We don't allow changes in Vcc. */
+    if (s->socket.Vcc != req->Vcc)
 	return CS_BAD_VCC;
-    
     if (req->Vpp1 != req->Vpp2)
 	return CS_BAD_VPP;
     s->socket.Vpp = req->Vpp1;
@@ -1541,7 +1538,7 @@ static int request_configuration(client_handle_t handle,
 	    if (!(c->irq.Attributes & IRQ_FORCED_PULSE))
 		c->Option |= COR_LEVEL_REQ;
 	write_cis_mem(s, 1, (base + CISREG_COR)>>1, 1, &c->Option);
-	udelay(40000);
+	udelay(40*1000);
     }
     if (req->Present & PRESENT_STATUS) {
 	c->Status = req->Status;
@@ -1778,6 +1775,8 @@ static int request_window(client_handle_t *handle, win_req_t *req)
     win->base = req->Base;
     win->size = req->Size;
     if (find_mem_region(&win->base, win->size, (*handle)->dev_info,
+			((s->cap.features & SS_CAP_MEM_ALIGN) ?
+			 req->Size : s->cap.map_size),
 			(req->Attributes & WIN_MAP_BELOW_1MB) ||
 			!(s->cap.features & SS_CAP_PAGE_REGS)))
 	return CS_IN_USE;
@@ -2145,8 +2144,10 @@ static struct symbol_table cs_symtab = {
     X(unregister_ss_entry),
     X(CardServices),
     X(MTDHelperEntry),
-    X(register_mem_region),
+#ifndef HAVE_MEMRESERVE
+    X(request_mem_region),
     X(release_mem_region),
+#endif
 #include <linux/symtab_end.h>
 };
 
@@ -2157,8 +2158,10 @@ EXPORT_SYMBOL(register_ss_entry);
 EXPORT_SYMBOL(unregister_ss_entry);
 EXPORT_SYMBOL(CardServices);
 EXPORT_SYMBOL(MTDHelperEntry);
-EXPORT_SYMBOL(register_mem_region);
+#ifndef HAVE_MEMRESERVE
+EXPORT_SYMBOL(request_mem_region);
 EXPORT_SYMBOL(release_mem_region);
+#endif
 
 #endif
 
@@ -2174,11 +2177,13 @@ int init_module(void)
     register_symtab(&cs_symtab);
 #ifdef HAS_PROC_BUS
     proc_pccard = create_proc_entry("pccard", S_IFDIR, proc_bus);
+#ifndef HAVE_MEMRESERVE
     if (proc_pccard) {
 	struct proc_dir_entry *ent;
 	ent = create_proc_entry("memory", 0, proc_pccard);
 	ent->read_proc = proc_read_mem;
     }
+#endif
 #endif
     return 0;
 }
