@@ -1,8 +1,8 @@
 /*
- * cs_internal.h 1.35 1999/01/07 03:52:53
+ * cs_internal.h 1.39 1999/07/20 16:01:25
  *
  * The contents of this file are subject to the Mozilla Public License
- * Version 1.0 (the "License"); you may not use this file except in
+ * Version 1.1 (the "License"); you may not use this file except in
  * compliance with the License. You may obtain a copy of the License
  * at http://www.mozilla.org/MPL/
  *
@@ -126,7 +126,7 @@ typedef struct config_t {
 #define MAX_CIS_DATA	512
 
 typedef struct socket_info_t {
-#if defined(CONFIG_SMP) && defined(SPIN_LOCK_UNLOCKED)
+#ifdef USE_SPIN_LOCKS
     spinlock_t			lock;
 #endif
     ss_entry_t			ss_entry;
@@ -149,7 +149,6 @@ typedef struct socket_info_t {
     cb_bridge_map		cb_cis_map;
     u_char			*cb_cis_virt;
     struct cb_config_t		*cb_config;
-    struct pci_bus		*pci_bus;
 #endif
     struct {
 	u_int			AssignedIRQ;
@@ -253,7 +252,7 @@ void validate_mem(int (*is_valid)(u_long), int (*do_cksum)(u_long),
 		  int force_low);
 int find_io_region(ioaddr_t *base, ioaddr_t num, char *name);
 int find_mem_region(u_long *base, u_long num, char *name,
-		    int force_low);
+		    u_long align, int force_low);
 int try_irq(u_int Attributes, int irq, int specific);
 void undo_irq(u_int Attributes, int irq);
 int adjust_resource_info(client_handle_t handle, adjust_t *adj);
