@@ -3,10 +3,10 @@
     Device driver for Intel 82365 and compatible PC Card controllers,
     and Yenta-compatible PCI-to-CardBus controllers.
 
-    i82365.c 1.236 1999/05/14 17:36:21
+    i82365.c 1.246 1999/07/20 16:01:27
 
     The contents of this file are subject to the Mozilla Public
-    License Version 1.0 (the "License"); you may not use this file
+    License Version 1.1 (the "License"); you may not use this file
     except in compliance with the License. You may obtain a copy of
     the License at http://www.mozilla.org/MPL/
 
@@ -72,7 +72,7 @@ MODULE_PARM(pc_debug, "i");
 #endif
 #define DEBUG(n, args) do { if (pc_debug>(n)) _printk args; } while (0)
 static const char *version =
-"i82365.c 1.236 1999/05/14 17:36:21 (David Hinds)";
+"i82365.c 1.246 1999/07/20 16:01:27 (David Hinds)";
 #else
 #define DEBUG(n, args) do { } while (0)
 #endif
@@ -179,6 +179,7 @@ static int pci_latency = -1;
 static int cb_latency = -1;
 static int cb_bus_base = 0;
 static int cb_bus_step = 2;
+static int cb_write_post = -1;
 MODULE_PARM(do_pci_probe, "i");
 MODULE_PARM(cb_mem_base, "i");
 MODULE_PARM(fast_pci, "i");
@@ -190,6 +191,7 @@ MODULE_PARM(pci_latency, "i");
 MODULE_PARM(cb_latency, "i");
 MODULE_PARM(cb_bus_base, "i");
 MODULE_PARM(cb_bus_step, "i");
+MODULE_PARM(cb_write_post, "i");
 #endif
 
 #ifdef CONFIG_ISA
@@ -226,7 +228,7 @@ typedef struct vg46x_state_t {
 
 typedef struct ti113x_state_t {
     u_int		sysctl;
-    u_char		cardctl, devctl;
+    u_char		cardctl, devctl, diag;
 } ti113x_state_t;
 
 typedef struct rl5c4xx_state_t {
@@ -256,7 +258,7 @@ typedef struct socket_info_t {
 #endif
 #ifdef CONFIG_PCI
     u_short		vendor, device;
-    u_char		bus, devfn;
+    u_char		revision, bus, devfn;
     u_short		bcr;
     u_char		pci_lat, cb_lat, sub_bus;
     u_char		cache, pmcs;
@@ -294,7 +296,7 @@ static void pcic_proc_remove(u_short sock);
 
 #ifdef CONFIG_ISA
 static int grab_irq;
-#if defined(CONFIG_SMP) && defined(SPIN_LOCK_UNLOCKED)
+#ifdef USE_SPIN_LOCKS
 static spinlock_t isa_lock = SPIN_LOCK_UNLOCKED;
 #endif
 #endif
@@ -359,11 +361,12 @@ typedef enum pcic_id {
 #define IS_VIA		0x0010
 #define IS_TOPIC	0x0020
 #define IS_RICOH	0x0040
-#define IS_UNKNOWN	0x0800
-#define IS_VG_PWR	0x1000
-#define IS_DF_PWR	0x2000
-#define IS_PCI		0x4000
-#define IS_CARDBUS	0x8000
+#define IS_UNKNOWN	0x0400
+#define IS_VG_PWR	0x0800
+#define IS_DF_PWR	0x1000
+#define IS_PCI		0x2000
+#define IS_CARDBUS	0x4000
+#define IS_ALIVE	0x8000
 
 typedef struct pcic_t {
     char		*name;
@@ -719,6 +722,7 @@ static void ti113x_get_state(u_short s)
     pci_readl(t->bus, t->devfn, TI113X_SYSTEM_CONTROL, &p->sysctl);
     pci_readb(t->bus, t->devfn, TI113X_CARD_CONTROL, &p->cardctl);
     pci_readb(t->bus, t->devfn, TI113X_DEVICE_CONTROL, &p->devctl);
+    pci_readb(t->bus, t->devfn, TI1250_DIAGNOSTIC, &p->diag);
 }
 
 static void ti113x_set_state(u_short s)
@@ -729,6 +733,7 @@ static void ti113x_set_state(u_short s)
     pci_writeb(t->bus, t->devfn, TI113X_CARD_CONTROL, p->cardctl);
     pci_writeb(t->bus, t->devfn, TI113X_DEVICE_CONTROL, p->devctl);
     pci_writeb(t->bus, t->devfn, TI1250_MULTIMEDIA_CTL, 0);
+    pci_writeb(t->bus, t->devfn, TI1250_DIAGNOSTIC, p->diag);
     i365_set_pair(s, TI113X_IO_OFFSET(0), 0);
     i365_set_pair(s, TI113X_IO_OFFSET(1), 0);
 }
@@ -745,6 +750,12 @@ static int ti113x_set_irq_mode(u_short s, int pcsc, int pint)
 	    p->cardctl |= TI113X_CCR_PCI_IRQ_ENA | TI113X_CCR_PCI_CSC;
 	if (pint)
 	    p->cardctl |= TI113X_CCR_PCI_IRQ_ENA | TI113X_CCR_PCI_IREQ;
+    } else if (t->type == IS_TI1250A) {
+	p->diag &= TI1250_DIAG_PCI_CSC | TI1250_DIAG_PCI_IREQ;
+	if (pcsc)
+	    p->diag |= TI1250_DIAG_PCI_CSC;
+	if (pint)
+	    p->diag |= TI1250_DIAG_PCI_IREQ;
     }
     return 0;
 }
@@ -769,6 +780,10 @@ static u_int ti113x_set_opts(u_short s, char *buf)
 	p->devctl &= ~TI113X_DCR_IMODE_MASK;
 	p->devctl |= TI113X_DCR_IMODE_SERIAL;
 	break;
+    case 3:
+	p->devctl &= ~TI113X_DCR_IMODE_MASK;
+	p->devctl |= TI12XX_DCR_IMODE_ALL_SERIAL;
+	break;
     default:
 	if ((p->devctl & TI113X_DCR_IMODE_MASK) == 0)
 	    p->devctl |= TI113X_DCR_IMODE_ISA;
@@ -1090,6 +1105,10 @@ static void cb_set_opts(u_short s, char *buf)
 {
     socket_info_t *t = &socket[s];
     t->bcr |= CB_BCR_WRITE_POST;
+    /* some TI1130's seem to exhibit problems with write posting */
+    if (((t->type == IS_TI1130) && (t->revision == 4) &&
+	 (cb_write_post < 0)) || (cb_write_post == 0))
+	t->bcr &= ~CB_BCR_WRITE_POST;
     if (t->cache == 0) t->cache = 8;
     if (pci_latency >= 0) t->pci_lat = pci_latency;
     if (t->pci_lat == 0) t->pci_lat = 0xa8;
@@ -1182,6 +1201,10 @@ static u_int set_host_opts(u_short s, u_short ns)
     char buf[128];
 
     for (i = s; i < s+ns; i++) {
+	if (socket[i].flags & IS_ALIVE) {
+	    printk(KERN_INFO "    host opts [%d]: already alive!\n", i);
+	    continue;
+	}
 	buf[0] = '\0';
 	get_host_state(i);
 	if (socket[i].flags & IS_CIRRUS)
@@ -1258,6 +1281,7 @@ static u_int test_irq(u_short sock, int irq, int pci)
     schedule_timeout(HZ/100);
     if (irq_hits) {
 	_free_irq(irq, irq_count);
+	DEBUG(2, ("    spurious hit!\n"));
 	return 1;
     }
 
@@ -1284,6 +1308,7 @@ static u_int test_irq(u_short sock, int irq, int pci)
 
     /* mask all interrupts */
     i365_set(sock, I365_CSCINT, 0);
+    DEBUG(2, ("    hits = %d\n", irq_hits));
     
     return (irq_hits != 1);
 }
@@ -1446,6 +1471,33 @@ static int identify(u_short port, u_short sock)
 
 #endif
 
+/*======================================================================
+
+    See if a card is present, powered up, in IO mode, and already
+    bound to a (non PC Card) Linux driver.  We leave these alone.
+
+    We make an exception for cards that seem to be serial devices.
+    
+======================================================================*/
+
+static int is_alive(u_short sock)
+{
+    u_char stat;
+    u_short start, stop;
+    
+    stat = i365_get(sock, I365_STATUS);
+    start = i365_get_pair(sock, I365_IO(0)+I365_W_START);
+    stop = i365_get_pair(sock, I365_IO(0)+I365_W_STOP);
+    if ((stat & I365_CS_DETECT) && (stat & I365_CS_POWERON) &&
+	(i365_get(sock, I365_INTCTL) & I365_PC_IOCARD) &&
+	(i365_get(sock, I365_ADDRWIN) & I365_ENA_IO(0)) &&
+	(check_region(start, stop-start+1) != 0) &&
+	((start & 0xfeef) != 0x02e8))
+	return 1;
+    else
+	return 0;
+}
+
 /*====================================================================*/
 
 static void add_socket(u_short port, int psock, int type)
@@ -1454,13 +1506,15 @@ static void add_socket(u_short port, int psock, int type)
     socket[sockets].psock = psock;
     socket[sockets].type = type;
     socket[sockets].flags = pcic[type].flags;
+    if (is_alive(sockets))
+	socket[sockets].flags |= IS_ALIVE;
     sockets++;
 }
 
 static void add_pcic(int ns, int type)
 {
     u_int mask = 0, i, base;
-    int pci_irq = 0, isa_irq = 0;
+    int use_pci = 0, isa_irq = 0;
     socket_info_t *t = &socket[sockets-ns];
 
     base = sockets-ns;
@@ -1504,20 +1558,22 @@ static void add_pcic(int ns, int type)
     if (pci_csc && t->cap.pci_irq) {
 	for (i = 0; i < ns; i++)
 	    if (_check_irq(t[i].cap.pci_irq, SA_SHIRQ)) break;
-	pci_irq = (i == ns);
-	printk(" PCI status changes\n");
+	if (i == ns) {
+	    use_pci = 1;
+	    printk(" PCI status changes\n");
+	}
     }
 #endif
     
 #ifdef CONFIG_ISA
     /* Poll if only two interrupts available */
-    if (!pci_irq && !poll_interval) {
+    if (!use_pci && !poll_interval) {
 	u_int tmp = (mask & (mask-1));
 	if ((tmp & (tmp-1)) == 0)
 	    poll_interval = HZ;
     }
     /* Only try an ISA cs_irq if this is the first controller */
-    if (!pci_irq && !grab_irq && (cs_irq || !poll_interval)) {
+    if (!use_pci && !grab_irq && (cs_irq || !poll_interval)) {
 	/* Avoid irq 12 unless it is explicitly requested */
 	u_int cs_mask = mask & ((cs_irq) ? (1<<cs_irq) : ~(1<<12));
 	for (cs_irq = 15; cs_irq > 0; cs_irq--)
@@ -1532,7 +1588,7 @@ static void add_pcic(int ns, int type)
     }
 #endif
     
-    if (!pci_irq && !isa_irq) {
+    if (!use_pci && !isa_irq) {
 	if (poll_interval == 0)
 	    poll_interval = HZ;
 	printk(" polling interval = %d ms\n",
@@ -1559,37 +1615,6 @@ static void add_pcic(int ns, int type)
 
 } /* add_pcic */
 
-/*======================================================================
-
-    See if a card is present, powered up, in IO mode, and already
-    bound to a (non PC Card) Linux driver.  We leave these alone.
-
-    We make an exception for cards that seem to be serial devices.
-    
-======================================================================*/
-
-static int is_active(u_short port, u_short sock)
-{
-    u_char stat;
-    u_short start, stop;
-    
-    /* Use the next free entry in the socket table */
-    socket[sockets].ioaddr = port;
-    socket[sockets].psock = sock;
-    
-    stat = i365_get(sockets, I365_STATUS);
-    start = i365_get_pair(sockets, I365_IO(0)+I365_W_START);
-    stop = i365_get_pair(sockets, I365_IO(0)+I365_W_STOP);
-    if ((stat & I365_CS_DETECT) && (stat & I365_CS_POWERON) &&
-	(i365_get(sockets, I365_INTCTL) & I365_PC_IOCARD) &&
-	(i365_get(sockets, I365_ADDRWIN) & I365_ENA_IO(0)) &&
-	(check_region(start, stop-start+1) != 0) &&
-	((start & 0xfeef) != 0x02e8))
-	return 1;
-    else
-	return 0;
-}
-
 /*====================================================================*/
 
 #ifdef CONFIG_PCI
@@ -1639,7 +1664,6 @@ static void add_pci_bridge(int type, u_char bus, u_char devfn,
     addr &= ~0x1;
     pci_writew(bus, devfn, PCI_COMMAND, CMD_DFLT);
     for (i = ns = 0; i < ((type == IS_I82092AA) ? 4 : 2); i++) {
-	if (is_active(addr, i)) continue;
 	s->bus = bus; s->devfn = devfn;
 	s->vendor = v; s->device = d;
 	add_socket(addr, i, type);
@@ -1653,8 +1677,8 @@ static void add_cb_bridge(int type, u_char bus, u_char devfn,
 {
     socket_info_t *s = &socket[sockets];
     u_short d, ns;
-    u_char a, b, max;
-
+    u_char a, b, r, max;
+    
     /* PCI bus enumeration is broken on some systems */
     for (ns = 0; ns < sockets; ns++)
 	if ((socket[ns].bus == bus) && (socket[ns].devfn == devfn))
@@ -1662,12 +1686,13 @@ static void add_cb_bridge(int type, u_char bus, u_char devfn,
     
     if (type == PCIC_COUNT) type = IS_UNK_CARDBUS;
     pci_readb(bus, devfn, PCI_HEADER_TYPE, &a);
+    pci_readb(bus, devfn, PCI_CLASS_REVISION, &r);
     max = (a & 0x80) ? 8 : 1;
     for (ns = 0; ns < max; ns++, s++, devfn++) {
 	if (pci_readw(bus, devfn, PCI_DEVICE_ID, &d) || (d != d0))
 	    break;
 	s->bus = bus; s->devfn = devfn;
-	s->vendor = v; s->device = d;
+	s->vendor = v; s->device = d; s->revision = r;
 	
 	/* Check for power management capabilities */
 	pci_readb(bus, devfn, PCI_STATUS, &a);
@@ -1694,22 +1719,45 @@ static void add_cb_bridge(int type, u_char bus, u_char devfn,
 	    pci_writel(bus, devfn, PCI_BASE_ADDRESS_0, s->cb_phys);
 	    cb_mem_base += PAGE_SIZE;
 	}
-	
+
+	request_mem_region(s->cb_phys, 0x1000, "i82365");
 	s->cb_virt = ioremap(s->cb_phys, 0x1000);
 	if (s->cb_virt == NULL)
 	    printk(KERN_NOTICE "i82365: ioremap failed: bad kernel!\n");
 	add_socket(0, 0, type);
     }
+    s -= ns;
     if (ns == 2) {
 	/* Nasty special check for bad bus mapping */
-	pci_readb(s[-2].bus, s[-2].devfn, CB_CARDBUS_BUS, &a);
-	pci_readb(s[-1].bus, s[-1].devfn, CB_CARDBUS_BUS, &b);
+	pci_readb(bus, s[0].devfn, CB_CARDBUS_BUS, &a);
+	pci_readb(bus, s[1].devfn, CB_CARDBUS_BUS, &b);
 	if (a == b) {
-	    pci_writeb(s[-2].bus, s[-2].devfn, CB_CARDBUS_BUS, 0);
-	    pci_writeb(s[-1].bus, s[-1].devfn, CB_CARDBUS_BUS, 0);
+	    pci_writeb(bus, s[0].devfn, CB_CARDBUS_BUS, 0);
+	    pci_writeb(bus, s[1].devfn, CB_CARDBUS_BUS, 0);
 	}
     }
     if (ns > 0) add_pcic(ns, type);
+
+#if (LINUX_VERSION_CODE >= VERSION(2,1,103))
+    /* Set up PCI bus bridge structures if needed */
+    for (a = 0; a < ns; a++) {
+	struct pci_dev *self = pci_find_slot(bus, s[a].devfn);
+	struct pci_bus *child, *parent = self->bus;
+	for (child = parent->children; child; child = child->next)
+	    if (child->number == s[a].cap.cardbus) break;
+	if (!child) {
+	    child = kmalloc(sizeof(struct pci_bus), GFP_KERNEL);
+	    memset(child, 0, sizeof(struct pci_bus));
+	    child->self = self;
+	    child->primary = bus;
+	    child->number = child->secondary = s[a].cap.cardbus;
+	    child->subordinate = s[a].sub_bus;
+	    child->parent = parent;
+	    child->next = parent->children;
+	}
+	s[a].cap.cb_bus = parent->children = child;
+    }
+#endif
 }
 
 static void pci_probe(u_int class, void (add_fn)(int, u_char, u_char,
@@ -1756,8 +1804,7 @@ static void isa_probe(void)
 	    if (i == ignore) continue;
 	    port = i365_base + ((i & 1) << 2) + ((i & 2) << 1);
 	    sock = (i & 1) << 1;
-	    if ((identify(port, sock) == IS_I82365DF) &&
-		!is_active(port, sock)) {
+	    if (identify(port, sock) == IS_I82365DF) {
 		add_socket(port, sock, IS_VLSI);
 		add_pcic(1, IS_VLSI);
 	    }
@@ -1771,8 +1818,7 @@ static void isa_probe(void)
 
 	    for (j = ns = 0; j < 2; j++) {
 		/* Does the socket exist? */
-		if ((ignore == i+j) || (identify(port, sock+j) < 0) ||
-		    is_active(port, sock+j))
+		if ((ignore == i+j) || (identify(port, sock+j) < 0))
 		    continue;
 		/* Check for bad socket decode */
 		for (k = 0; k <= sockets; k++)
@@ -1881,8 +1927,10 @@ static void pcic_finish(void)
 	i365_set(i, I365_CSCINT, 0);
 	release_region(socket[i].ioaddr, 2);
 #ifdef CONFIG_PCI
-	if (socket[i].cb_virt)
+	if (socket[i].cb_virt) {
 	    iounmap(socket[i].cb_virt);
+	    release_mem_region(socket[i].cb_phys, 0x1000);
+	}
 #endif
     }
 } /* pcic_finish */
@@ -2045,7 +2093,8 @@ static int i365_get_socket(u_short sock, socket_state_t *state)
     vcc = reg & I365_VCC_MASK; vpp = reg & I365_VPP1_MASK;
     state->Vcc = state->Vpp = 0;
 #ifdef CONFIG_PCI
-    if ((t->flags & IS_CARDBUS) && !(t->flags & IS_TOPIC)) {
+    if ((t->flags & IS_CARDBUS) &&
+	(!(t->flags & IS_TOPIC) || (t->type == IS_TOPIC97))) {
 	cb_get_power(sock, state);
     } else
 #endif
@@ -2146,7 +2195,8 @@ static int i365_set_socket(u_short sock, socket_state_t *state)
     if (state->flags & SS_OUTPUT_ENA) reg |= I365_PWR_OUT;
 
 #ifdef CONFIG_PCI
-    if ((t->flags & IS_CARDBUS) && !(t->flags & IS_TOPIC)) {
+    if ((t->flags & IS_CARDBUS) &&
+	(!(t->flags & IS_TOPIC) || (t->type == IS_TOPIC97))) {
 	cb_set_power(sock, state);
 	reg |= i365_get(sock, I365_POWER) &
 	    (I365_VCC_MASK|I365_VPP1_MASK);
@@ -2740,6 +2790,12 @@ static int pcic_service(u_int sock, u_int cmd, void *arg)
     if (cmd >= NFUNC)
 	return -EINVAL;
 
+    if (socket[sock].flags & IS_ALIVE) {
+	if (cmd == SS_GetStatus)
+	    *(u_int *)arg = 0;
+	return -EINVAL;
+    }
+    
     fn = pcic_service_table[cmd];
 #ifdef CONFIG_CARDBUS
     if ((socket[sock].flags & IS_CARDBUS) &&
