@@ -2,10 +2,10 @@
 
     Resource management routines
 
-    rsrc_mgr.c 1.59 1999/02/08 07:04:30
+    rsrc_mgr.c 1.67 1999/07/30 03:48:44
 
     The contents of this file are subject to the Mozilla Public
-    License Version 1.0 (the "License"); you may not use this file
+    License Version 1.1 (the "License"); you may not use this file
     except in compliance with the License. You may obtain a copy of
     the License at http://www.mozilla.org/MPL/
 
@@ -89,7 +89,7 @@ static irq_info_t irq_table[NR_IRQS] = { { 0, 0, 0 }, /* etc */ };
 
 #endif
 
-#if defined(CONFIG_SMP) && defined(SPIN_LOCK_UNLOCKED)
+#ifdef USE_SPIN_LOCKS
 static spinlock_t rsrc_lock = SPIN_LOCK_UNLOCKED;
 #endif
 
@@ -100,7 +100,7 @@ static spinlock_t rsrc_lock = SPIN_LOCK_UNLOCKED;
     
 ======================================================================*/
 
-#ifdef __LINUX__
+#if defined(__LINUX__) && !defined(HAVE_MEMRESERVE)
 
 typedef struct resource_entry_t {
     u_long			base, num;
@@ -130,8 +130,8 @@ static resource_entry_t *find_gap(resource_entry_t *root,
     return p;
 }
 
-static int register_resource(resource_entry_t *list,
-			     u_long base, u_long num, char *name)
+static int register_my_resource(resource_entry_t *list,
+				u_long base, u_long num, char *name)
 {
     u_long flags;
     resource_entry_t *p, *entry;
@@ -154,8 +154,8 @@ static int register_resource(resource_entry_t *list,
     return 0;
 }
 
-static int release_resource(resource_entry_t *list,
-			    u_long base, u_long num)
+static void release_my_resource(resource_entry_t *list,
+				u_long base, u_long num)
 {
     u_long flags;
     resource_entry_t *p, *q;
@@ -168,36 +168,36 @@ static int release_resource(resource_entry_t *list,
 	    p->next = q->next;
 	    kfree_s(q, sizeof(resource_entry_t));
 	    spin_unlock_irqrestore(&rsrc_lock, flags);
-	    return 0;
+	    return;
 	}
     }
     spin_unlock_irqrestore(&rsrc_lock, flags);
-    return -EINVAL;
+    return;
 }
 
-static int check_resource(resource_entry_t *list,
-			  u_long base, u_long num)
+static int check_my_resource(resource_entry_t *list,
+			     u_long base, u_long num)
 {
-    if (register_resource(list, base, num, NULL) != 0)
+    if (register_my_resource(list, base, num, NULL) != 0)
 	return -EBUSY;
-    release_resource(list, base, num);
+    release_my_resource(list, base, num);
     return 0;
 }
 
 int check_mem_region(u_long base, u_long num)
 {
-    return check_resource(&mem_list, base, num);
+    return check_my_resource(&mem_list, base, num);
 }
-int register_mem_region(u_long base, u_long num, char *name)
+void request_mem_region(u_long base, u_long num, char *name)
 {
-    return register_resource(&mem_list, base, num, name);
+    register_my_resource(&mem_list, base, num, name);
 }
-int release_mem_region(u_long base, u_long num)
+void release_mem_region(u_long base, u_long num)
 {
-    return release_resource(&mem_list, base, num);
+    release_my_resource(&mem_list, base, num);
 }
 
-#endif /* __LINUX__ */
+#endif /* __LINUX__ && !HAVE_MEMRESOURCE */
 
 /*======================================================================
 
@@ -267,7 +267,7 @@ int check_resource(int type, u_long base, u_long num)
 
 /*====================================================================*/
 
-#ifdef HAS_PROC_BUS
+#if defined(HAS_PROC_BUS) && !defined(HAVE_MEMRESERVE)
 int proc_read_mem(char *buf, char **start, off_t pos,
 		  int count, int *eof, void *data)
 {
@@ -456,10 +456,28 @@ static int do_mem_probe(u_long base, u_long num,
 
 #ifdef CONFIG_ISA
 
+static u_long inv_probe(int (*is_valid)(u_long),
+			int (*do_cksum)(u_long),
+			resource_map_t *m)
+{
+    u_long ok;
+    if (m == &mem_db)
+	return 0;
+    ok = inv_probe(is_valid, do_cksum, m->next);
+    if (ok) {
+	if (m->base >= 0x100000)
+	    sub_interval(&mem_db, m->base, m->num);
+	return ok;
+    }
+    if (m->base < 0x100000)
+	return 0;
+    return do_mem_probe(m->base, m->num, is_valid, do_cksum);
+}
+
 void validate_mem(int (*is_valid)(u_long), int (*do_cksum)(u_long),
 		  int force_low)
 {
-    resource_map_t *m;
+    resource_map_t *m, *n;
     static u_char order[] = { 0xd0, 0xe0, 0xc0, 0xf0 };
     static int hi = 0, lo = 0;
     u_long b, i, ok = 0;
@@ -467,18 +485,14 @@ void validate_mem(int (*is_valid)(u_long), int (*do_cksum)(u_long),
     if (!probe_mem) return;
     /* We do up to four passes through the list */
     if (!force_low) {
-	if (hi++) return;
-	for (m = mem_db.next; m != &mem_db; m = m->next) {
-	    /* Only probe > 1 MB */
-	    if (m->base < 0x100000) continue;
-	    ok += do_mem_probe(m->base, m->num, is_valid, do_cksum);
-	    if (ok) return;
-	}
+	if (hi++ || (inv_probe(is_valid, do_cksum, mem_db.next) > 0))
+	    return;
 	printk(KERN_NOTICE "cs: warning: no high memory space "
 	       "available!\n");
     }
     if (lo++) return;
-    for (m = mem_db.next; m != &mem_db; m = m->next) {
+    for (m = mem_db.next; m != &mem_db; m = n) {
+	n = m->next;
 	/* Only probe < 1 MB */
 	if (m->base >= 0x100000) continue;
 	if ((m->base | m->num) & 0xffff) {
@@ -558,29 +572,30 @@ int find_io_region(ioaddr_t *base, ioaddr_t num, char *name)
 } /* find_io_region */
 
 int find_mem_region(u_long *base, u_long num, char *name,
-		    int force_low)
+		    u_long align, int force_low)
 {
-    u_long align;
     resource_map_t *m;
 
     if (*base != 0) {
 	for (m = mem_db.next; m != &mem_db; m = m->next) {
 	    if ((*base >= m->base) && (*base+num <= m->base+m->num))
-		return register_mem_region(*base, num, name);
+		if (check_mem_region(*base, num) == 0) {
+		    request_mem_region(*base, num, name);
+		    return 0;
+		}
 	}
 	return -1;
     }
     
-    for (align = 4096; align < num; align *= 2) ;
-
     ACQUIRE_RESOURCE_LOCK;
     while (1) {
 	for (m = mem_db.next; m != &mem_db; m = m->next) {
 	    /* first pass >1MB, second pass <1MB */
 	    if ((force_low != 0) ^ (m->base < 0x100000)) continue;
 	    for (*base = (m->base + align - 1) & (~(align-1));
-		 *base+align <= m->base+m->num; *base += align)
-		if (register_mem_region(*base, num, name) == 0) {
+		 *base+num <= m->base+m->num; *base += align)
+		if (check_mem_region(*base, num) == 0) {
+		    request_mem_region(*base, num, name);
 		    RELEASE_RESOURCE_LOCK;
 		    return 0;
 		}
