@@ -1,8 +1,8 @@
 /*
- * rsrc_mgr.h 1.14 1998/10/01 09:16:06
+ * rsrc_mgr.h 1.17 1999/07/20 16:01:30
  *
  * The contents of this file are subject to the Mozilla Public License
- * Version 1.0 (the "License"); you may not use this file except in
+ * Version 1.1 (the "License"); you may not use this file except in
  * compliance with the License. You may obtain a copy of the License
  * at http://www.mozilla.org/MPL/
  *
@@ -19,12 +19,6 @@
 #ifndef _RSRC_MGR_H
 #define _RSRC_MGR_H
 
-#ifdef __LINUX__
-int check_mem_region(u_long base, u_long num);
-int register_mem_region(u_long base, u_long num, char *name);
-int release_mem_region(u_long base, u_long num);
-#endif
-
 #ifdef __BEOS__
 int check_resource(int type, u_long base, u_long num);
 int register_resource(int type, u_long base, u_long num);
