@@ -2,10 +2,10 @@
 
     Device driver for Databook TCIC-2 PCMCIA controller
 
-    tcic.c 1.100 1999/02/25 17:08:09
+    tcic.c 1.102 1999/07/20 16:01:33
 
     The contents of this file are subject to the Mozilla Public
-    License Version 1.0 (the "License"); you may not use this file
+    License Version 1.1 (the "License"); you may not use this file
     except in compliance with the License. You may obtain a copy of
     the License at http://www.mozilla.org/MPL/
 
@@ -50,7 +50,7 @@
 static int pc_debug = PCMCIA_DEBUG;
 MODULE_PARM(pc_debug, "i");
 static const char *version =
-"tcic.c 1.100 1999/02/25 17:08:09 (David Hinds)";
+"tcic.c 1.102 1999/07/20 16:01:33 (David Hinds)";
 #define DEBUG(n, args...) if (pc_debug>(n)) printk(KERN_DEBUG args)
 #else
 #define DEBUG(n, args...)
@@ -116,9 +116,10 @@ static int sockets;
 static socket_info_t socket_table[2];
 
 static socket_cap_t tcic_cap = {
-    SS_CAP_PCCARD,	/* no special features */
+    /* only 16-bit cards, memory windows must be size-aligned */
+    SS_CAP_PCCARD | SS_CAP_MEM_ALIGN,
     0x4cf8,		/* irq 14, 11, 10, 7, 6, 5, 4, 3 */
-    0x1000,		/* 4K memory windows */
+    0x1000,		/* 4K minimum window size */
     0, 0		/* No PCI or CardBus support */
 };
 
